@@ -5,12 +5,15 @@ ConvNet Inference of an image chunk
 import os
 import logging
 import time
-import numpy as np
-from tqdm import tqdm
 from warnings import warn
 from typing import Union
+
+import numpy as np
+from tqdm import tqdm
+
+from chunkflow.lib.cartesian_coordinate import Cartesian, to_cartesian
+
 from .patch.base import PatchInferencerBase
-from tempfile import mktemp
 from chunkflow.chunk import Chunk
 # from chunkflow.chunk.affinity_map import AffinityMap
 
@@ -33,29 +36,61 @@ class Inferencer(object):
     def __init__(self,
                  convnet_model: Union[str, PatchInferencerBase],
                  convnet_weight_path: str,
-                 input_patch_size: Union[tuple, list],
-                 output_patch_size: Union[tuple, list] = None,
-                 patch_num: Union[tuple, list] = None,
+                 input_patch_size: Union[tuple, list, Cartesian],
+                 output_patch_size: Union[tuple, list, Cartesian] = None,
+                 patch_num: Union[tuple, list, Cartesian] = None,
                  num_output_channels: int = 3,
-                 output_patch_overlap: Union[tuple, list] = (4, 64, 64),
-                 output_crop_margin: Union[tuple, list] = None,
+                 output_patch_overlap: Union[tuple, list, Cartesian] = None,
+                 output_crop_margin: Union[tuple, list, Cartesian] = None,
                  dtype = 'float32',
-                 framework: str = 'identity',
+                 framework: str = 'universal',
                  batch_size: int = 1,
                  bump: str = 'wu',
-                 input_size: tuple = None,
-                 mask_output_chunk: bool = False,
+                 input_size: Union[tuple, list, Cartesian] = None,
+                 mask_output_chunk: bool = True,
                  mask_myelin_threshold = None,
+                 test_time_augmentation: bool = False,
                  dry_run: bool = False):
-        assert input_size is None or patch_num is None 
-        
-        if output_patch_size is None:
-            output_patch_size = input_patch_size 
+        """convnet inference patch by patch in a chunk
 
+        Args:
+            convnet_model (Union[str, PatchInferencerBase]): the path of convnet model
+            convnet_weight_path (str): the path of trained model weights
+            input_patch_size (Union[tuple, list, Cartesian]): input patch size, zyx
+            output_patch_size (Union[tuple, list, Cartesian], optional): output patch size. Defaults to the same with input patch size.
+            patch_num (Union[tuple, list, Cartesian], optional): number of patches. Defaults to be computed.
+            num_output_channels (int, optional): number of output channels. Defaults to 3.
+            output_patch_overlap (Union[tuple, list, Cartesian], optional): the overlap size of output patch size. Defaults to be half of output patch size.
+            output_crop_margin (Union[tuple, list, Cartesian], optional): crop some output patch margin. Defaults to None.
+            dtype (str, optional): data type named consistantly with numpy. Defaults to 'float32'.
+            framework (str, optional): ['universal', 'identity', 'pytorch']. Defaults to 'universal'.
+            batch_size (int, optional): batch size in one pass. this parameter seems do not accelarate computation. Defaults to 1.
+            bump (str, optional): bump function. Defaults to 'wu'.
+            input_size (Union[tuple, list, Cartesian], optional): input chunk size. Defaults to None.
+            mask_output_chunk (bool, optional): normalize on the chunk level rather than patch level. Defaults to True.
+            mask_myelin_threshold (_type_, optional): threshold to segment the myelin. Defaults to None.
+            test_time_augmentation (bool, optional): augment the image patch, inference, transform back and blend. Defaults to True.
+            dry_run (bool, optional): only compute parameters and setup, do not perform any real computation. Defaults to False.
+        """
+        assert input_size is None or patch_num is None 
+         
         if logging.getLogger().getEffectiveLevel() <= 30:
             self.verbose = True
         else:
             self.verbose = False
+
+        input_patch_size = to_cartesian(input_patch_size)
+        patch_num = to_cartesian(patch_num)
+        input_size = to_cartesian(input_size)
+        output_patch_size = to_cartesian(output_patch_size)
+        output_patch_overlap = to_cartesian(output_patch_overlap)
+        output_crop_margin = to_cartesian(output_crop_margin)
+
+        if output_patch_size is None:
+            output_patch_size = input_patch_size 
+
+        if output_patch_overlap is None:
+            output_patch_overlap = output_patch_size // 2
         
         self.input_patch_size = input_patch_size
         self.output_patch_size = output_patch_size
@@ -63,26 +98,23 @@ class Inferencer(object):
         self.patch_num = patch_num
         self.batch_size = batch_size
         self.input_size = input_size
-        
+
         if output_crop_margin is None:
             if mask_output_chunk:
-                self.output_crop_margin = (0,0,0)
+                self.output_crop_margin = Cartesian(0,0,0)
             else:
                 self.output_crop_margin = self.output_patch_overlap
         else:
             self.output_crop_margin = output_crop_margin
             # we should always crop more than the patch overlap 
             # since the overlap region is reweighted by patch mask
-            # To-Do: equal should also be OK
-            assert np.alltrue([v<=m for v, m in zip(
-                self.output_patch_overlap, 
-                self.output_crop_margin)])
+            assert self.output_crop_margin >= self.output_patch_overlap
 
-        self.output_patch_crop_margin = tuple((ips-ops)//2 for ips, ops in zip(
-            input_patch_size, output_patch_size))
+        self.output_patch_crop_margin = (input_patch_size - output_patch_size) // 2
         
-        self.output_offset = tuple(opcm+ocm for opcm, ocm in zip(
-            self.output_patch_crop_margin, self.output_crop_margin))
+        #self.output_offset = tuple(opcm+ocm for opcm, ocm in zip(
+        #    self.output_patch_crop_margin, self.output_crop_margin))
+        self.output_offset = self.output_crop_margin
     
         self.output_patch_stride = tuple(s - o for s, o in zip(
             output_patch_size, output_patch_overlap))
@@ -132,6 +164,8 @@ class Inferencer(object):
         if isinstance(convnet_weight_path, str):
             convnet_weight_path = os.path.expanduser(convnet_weight_path)
         self._prepare_patch_inferencer(framework, convnet_model, convnet_weight_path, bump)
+
+        self.test_time_augmentation = test_time_augmentation
    
     @property
     def compute_device(self):
@@ -149,7 +183,7 @@ class Inferencer(object):
         patch offset list and output chunk mask. Otherwise, recompute them.
         """
         if np.array_equal(self.input_size, input_chunk.shape):
-            print('reusing output chunk mask.')
+            logging.info('reusing output chunk mask.')
             assert self.patch_slices_list is not None
         else:
             if self.input_size is not None:
@@ -230,7 +264,7 @@ class Inferencer(object):
         input_patch_overlap = self.input_patch_overlap 
         input_patch_stride = self.input_patch_stride 
 
-        print('Construct patch slices list...')
+        logging.info('Construct patch slices list...')
         for iz in range(0, self.input_size[0] - input_patch_overlap[0], input_patch_stride[0]):
             if iz + input_patch_size[0] > self.input_size[0]:
                 iz = self.input_size[0] - input_patch_size[0]
@@ -298,7 +332,7 @@ class Inferencer(object):
         # this mask will result in 1
         self.output_chunk_mask.array = 1.0 / self.output_chunk_mask.array
     
-    def _get_output_buffer(self, input_chunk):
+    def _get_output_buffer(self, input_chunk: Chunk):
         output_buffer_size = (self.patch_inferencer.num_output_channels, ) + self.output_size
         #if self.mask_myelin_threshold is None:
         # a random temporal file. will be removed later.
@@ -314,7 +348,7 @@ class Inferencer(object):
         
         output_voxel_offset = tuple(io + ocso for io, ocso in zip(
             input_chunk.voxel_offset, self.output_offset))
-        
+
         output_buffer = Chunk(
             output_buffer_array,
             voxel_offset=output_voxel_offset,
@@ -335,7 +369,7 @@ class Inferencer(object):
             self._check_alignment()
          
         if self.dry_run:
-            print('dry run, return a special artifical chunk.')
+            logging.info('dry run, return a special artifical chunk.')
             size=output_buffer.shape
             
             if self.mask_myelin_threshold:
@@ -349,8 +383,8 @@ class Inferencer(object):
                 voxel_size=input_chunk.voxel_size,
             )
        
-        if input_chunk == 0:
-            print('input is all zero, return zero buffer directly')
+        if np.all(input_chunk == 0):
+            logging.info('input is all zero, return zero buffer directly')
             if self.mask_myelin_threshold:
                 assert output_buffer.shape[0] == 4
                 return output_buffer[:-1, ...]
@@ -377,18 +411,20 @@ class Inferencer(object):
                     batch_idx, 0, :, :, :] = input_chunk.cutout(slices[0]).array
 
             end = time.time()
-            logging.debug('prepare %d input patches takes %3f sec' %
-                    (self.batch_size, end - start))
+            logging.debug(f'prepare {self.batch_size:d} input patches takes {end-start:.3f} sec')
             start = end
 
             # the input and output patch is a 5d numpy array with
             # datatype of float32, the dimensions are batch/channel/z/y/x.
             # the input image should be normalized to [0,1]
-            output_patch = self.patch_inferencer(self.input_patch_buffer)
+            if not self.test_time_augmentation:
+                output_patch = self.patch_inferencer(self.input_patch_buffer)
+            else:
+                # test time augmentation
+                pass
 
             end = time.time()
-            logging.debug('run inference for %d patch takes %3f sec' %
-                    (self.batch_size, end - start))
+            logging.debug(f'run inference for {self.batch_size:d} patch takes {end-start:.3f} sec')
             start = end
 
             for batch_idx, slices in enumerate(batch_slices):
@@ -397,7 +433,7 @@ class Inferencer(object):
                 # the slices[0] is for input patch slice
                 # the slices[1] is for output patch slice
                 offset = tuple(s.start for s in slices[1])
-                output_chunk = Chunk(
+                output_patch_chunk = Chunk(
                     output_patch[batch_idx, :, :, :, :],
                     voxel_offset=offset,
                     voxel_size=input_chunk.voxel_size)
@@ -411,12 +447,12 @@ class Inferencer(object):
                 #    output_chunk.to_tif()
                 #    #input_chunk.cutout(slices[0]).to_tif()
 
-                output_buffer.blend(output_chunk)
+                output_buffer.blend(output_patch_chunk)
 
             end = time.time()
-            logging.debug('blend patch takes %3f sec' % (end - start))
-            logging.debug("Inference of whole chunk takes %3f sec" %
-                  (time.time() - chunk_time_start))
+            logging.debug('blend patch takes {:.3f} sec'.format(end - start))
+            logging.debug("Inference of whole chunk takes {:.3f} sec".format(
+                time.time() - chunk_time_start))
         
         if self.mask_output_chunk:
             output_buffer *= self.output_chunk_mask
