@@ -1,12 +1,12 @@
 #!/usr/bin/env python
 import sys
-from .base import PatchEngine
+import platform
+from .base import PatchInferencerBase
 
 
-class PZNet(PatchEngine):
-    def __init__(self, net_dir, use_bn=True, is_static_batch_norm=False,
-                 dtype: str='float32'):
-        super().__init__(dtype=dtype)
+class PZNet(PatchInferencerBase):
+    def __init__(self, net_dir):
+        super().__init__(dtype='float32')
         sys.path.append(net_dir)
         import pznet
         # self.net = pznet.znet(model_file_name, net_file_name)
