@@ -1,4 +1,7 @@
 __doc__ = """Image chunk class"""
+import os
+import json
+
 import logging
 import multiprocessing
 from typing import Union
@@ -8,26 +11,32 @@ from .base import Chunk
 
 # from ...lib.gala import evaluate
 from chunkflow.lib.gala import evaluate
+from chunkflow.lib.cartesian_coordinate import Cartesian
 
 import kimimaro
 import fastremap
 
-from cloudvolume.storage import Storage
+from cloudfiles import CloudFiles
 
 
 class Segmentation(Chunk):
     """
     a chunk of segmentation volume.
     """
-    def __init__(self, array, voxel_offset=None):
-        super().__init__(array, voxel_offset=voxel_offset)
+    def __init__(self, array: np.ndarray, 
+            voxel_offset: Cartesian=None, 
+            voxel_size:Cartesian=None):
+        super().__init__(array, voxel_offset=voxel_offset, 
+            voxel_size=voxel_size)
         assert array.ndim == 3
         assert np.issubdtype(array.dtype, np.integer)
 
     @classmethod
     def from_chunk(cls, chunk):
         assert isinstance(chunk, Chunk)
-        return cls(chunk.array, voxel_offset=chunk.voxel_offset)
+        return cls(
+            chunk.array, voxel_offset=chunk.voxel_offset, 
+            voxel_size = chunk.voxel_size)
 
     def evaluate(self, groundtruth, size_threshold: int=1000):
         """
@@ -85,8 +94,8 @@ class Segmentation(Chunk):
 
         if isinstance(selected_obj_ids, str) and selected_obj_ids.endswith('.json'):
             # assume that ids is a json file in the storage path
-            json_storage = Storage(os.path.dirname(selected_obj_ids))
-            ids_str = json_storage.get_file(os.path.basename(selected_obj_ids))
+            json_storage = CloudFiles(os.path.dirname(selected_obj_ids))
+            ids_str = json_storage.get(os.path.basename(selected_obj_ids))
             selected_obj_ids = set(json.loads(ids_str))
             assert len(selected_obj_ids) > 0
             logging.info(f'number of selected objects: {len(selected_obj_ids)}')
