@@ -34,9 +34,8 @@ class NeuroglancerOperator(OperatorBase):
         annotations = []
         
         pre_synapses = synapses.pre_with_physical_coordinate
-        self._append_point_annotation_layer(viewer_state, name + '_pre', pre_synapses)
-
         post_synapses = synapses.post_with_physical_coordinate
+        
         if post_synapses is not None:
             for post_idx in range(post_synapses.shape[0]):
                 pre_idx = post_synapses[post_idx, 0]
@@ -44,8 +43,6 @@ class NeuroglancerOperator(OperatorBase):
                 post_coordinate = post_synapses[post_idx, 1:]
                 post_annotation = ng.LineAnnotation(
                         id=str(post_idx),
-                        # note that the synapse coordinate is already in xyz order
-                        # so we do not need to reverse it!
                         pointA=pre_coordinate,
                         pointB=post_coordinate,
                         props=['#0ff', 5]
@@ -78,6 +75,10 @@ void main() {
             ),
         )
 
+        self._append_point_annotation_layer(viewer_state, name + '_pre', pre_synapses)
+
+
+
     def _append_point_annotation_layer(self, viewer_state: ng.viewer_state.ViewerState, name: str, points: np.ndarray):
         annotations = []
         
@@ -202,7 +203,7 @@ emitRGB(vec3(toNormalized(getDataValue(0)),
                 data=chunk.array,
                 dimensions=dimensions,
                 # offset is in nm, not voxels
-                voxel_offset=(0, ) + chunk.voxel_offset,
+                voxel_offset=(0, *chunk.voxel_offset),
             ),
             shader=shader
         )
