@@ -2,19 +2,31 @@ from __future__ import annotations
 
 import os
 import json
+import time
 from typing import List
 from copy import deepcopy
 
 import numpy as np
 import h5py
 
-from chunkflow.lib.bounding_boxes import BoundingBox, Cartesian
+from scipy.spatial import KDTree
+
+import chunkflow
+from chunkflow.chunk import Chunk
+from chunkflow.lib.cartesian_coordinate import BoundingBox, Cartesian
 
 
 class Synapses():
-    def __init__(self, pre: np.ndarray, pre_confidence: np.ndarray = None, 
-            post: np.ndarray = None, post_confidence: np.ndarray = None, 
-            resolution: Cartesian = None) -> None:
+    def __init__(self, 
+            pre: np.ndarray,
+            pre_confidence: np.ndarray = None, 
+            post: np.ndarray = None, 
+            post_confidence: np.ndarray = None, 
+            resolution: Cartesian = None,
+            users: list = None,
+            pre_users: np.ndarray = None,
+            post_users: np.ndarray = None,
+        ) -> None:
         """Synapses containing T-bars and post-synapses
 
         Args:
@@ -22,10 +34,13 @@ class Synapses():
             pre_confidence (np.ndarray, optional): confidence of T-bar detection. Defaults to None.
             post (np.ndarray, optional): [description]. Defaults to None.
             resolution (Cartesian, optional): [description]. Defaults to None.
+            users (list[str], optional): user names of edting/ingesting the synapses.
+            pre_user (np.ndarray(int8), optional): the user ids of presynapses. -1 means not identified.
+            post_user (np.ndarray(int8), optional): the user ids of postsynapses. -1 means not identified.
         """
         assert pre.ndim == 2
         assert pre.shape[1] == 3
-
+        
         if pre_confidence is not None:
             assert pre_confidence.ndim == 1
             np.testing.assert_array_less(pre_confidence, 1.00001)
@@ -35,7 +50,7 @@ class Synapses():
         if post is not None:
             if post_confidence is not None:
                 assert post_confidence.ndim == 1
-                assert len(post_confidence) == post.shape[1]
+                assert len(post_confidence) == post.shape[0]
 
             assert post.ndim == 2
             # parent pre index, z, y, x
@@ -45,6 +60,13 @@ class Synapses():
         if resolution is not None:
             resolution = np.asarray(resolution, dtype=pre.dtype)
             np.testing.assert_array_less(0, resolution)
+        
+        if users is not None:
+            assert isinstance(users, list)
+        if pre_users is not None:
+            assert pre_users.ndim == 1 and len(pre_users) == pre.shape[0]
+        if post_users is not None:
+            assert post_users.ndim == 1 and len(post_users) == post.shape[0]
 
         self.resolution = resolution
         # unsigned integer will have minus issues
@@ -55,23 +77,28 @@ class Synapses():
         self.post = post
         self.post_confidence = post_confidence
         
+        self.users = users
+        self.pre_users = pre_users
+        self.post_users = post_users
+
+        
     @classmethod
-    def from_dict(cls, dc: dict):
+    def from_dict(cls, syns_dict: dict):
         """Synapses as a dictionary
 
         Args:
-            synapses (dict): the whole synapses in a dictionary
+            syns_dict (dict): the whole synapses in a dictionary
         """
-        order = dc['order']
-        resolution = dc['resolution']
-        del dc['order']
-        del dc['resolution']
+        order = syns_dict['order']
+        resolution = syns_dict['resolution']
+        del syns_dict['order']
+        del syns_dict['resolution']
 
-        pre_num = len(synapses)
+        pre_num = len(syns_dict)
         pre = np.zeros((pre_num, 3), dtype=np.int32)
         post_list = []
         pre_indices = []
-        for sid, synapse in enumerate(synapses.values()):
+        for sid, synapse in enumerate(syns_dict.values()):
             pre[sid, :] = np.asarray(synapse['coord'])
             if 'postsynapses' in synapse:
                 for idx, post_coordinate in enumerate(synapse['postsynapses']):
@@ -112,40 +139,49 @@ class Synapses():
             syns = fivol.get_syndata(dvid_url, uuid)
             synapses = Synapses.from_dvid_list(syns)
         """
+        print(f'loading {len(syns)} synapses...')
         pre_list = []
-        pre_proofread = []
         post_list = []
         pre_confidence = []
+        pre_users = []
         for syn in syns:
             if 'Pre' in syn['Kind']:
                 # map from xyz to zyx
                 pos = syn['Pos'][::-1]
                 pos = Cartesian(*pos)
                 pre_list.append(pos)
-                if syn['Prop']['user'] == 'jwu':
-                    pre_proofread.append(False)
-                else:
-                    pre_proofread.append(True)
 
                 if 'conf' in syn['Prop']:
                     conf = syn['Prop']['conf']
                     conf = float(conf)
                 else:
-                    conf = 0.5
+                    conf = 1.0
                 pre_confidence.append(conf)
-            elif 'Post' in syn['Kind']:
+
+                user = syn['Prop']['user']
+                pre_users.append(user) 
+
+        print('loading post synapses...')
+        pre_set = set(pre_list)
+        post_users = []
+        for syn in syns:
+            if 'Post' in syn['Kind']:
                 # map from xyz to zyx
                 pos = syn['Pos'][::-1]
                 pos = Cartesian(*pos)
-                # if 'To' not in syn['Prop']:
-                    # print(syn)
-                    # breakpoint()
-                pre_pos = syn['Rels'][0]['To'][::-1]
-                pre_pos = Cartesian(*pre_pos)
-                post_list.append((pos, pre_pos))
-            else:
-                raise ValueError('unexpected synapse type: ', syn)
-        
+                if len(syn['Rels'])>0:
+                    pre_pos = syn['Rels'][0]['To'][::-1]
+                    pre_pos = Cartesian(*pre_pos)
+                    if pre_pos in pre_set:
+                        post_list.append((pos, pre_pos))
+                        user = syn['Prop']['user']
+                        post_users.append(user)
+                    else:
+                        print('found a postsynapse with deleted presynapse: ', syn)
+                else:
+                    print('found an post synapse without presynapse: ', syn)
+                
+                
         # build a map from pre position to index
         pre_pos2idx = {}
         for idx, pos in enumerate(pre_list):
@@ -165,23 +201,49 @@ class Synapses():
         post_list = np.asarray(post_list, dtype=np.int32)
         post_to_pre_indices = np.expand_dims(post_to_pre_indices, 1)
         post = np.hstack((post_to_pre_indices, post_list))
+
+        users = set(pre_users).union(set(post_users))
+        users = list(users)
+        user2id = {}
+        for idx, user in enumerate(users):
+            user2id[user] = idx
+        for idx, user in enumerate(pre_users):
+            pre_users[idx] = user2id[user]
+        for idx, user in enumerate(post_users):
+            post_users[idx] = user2id[user]
+
+        pre_users = np.asarray(pre_users, dtype=np.int32)
+        post_users = np.asarray(post_users, dtype=np.int32)
         return cls(
             pre, post=post, 
             pre_confidence=pre_confidence,
             resolution=resolution,
+            users = users,
+            pre_users = pre_users,
+            post_users = post_users,
         )
             
     @classmethod
-    def from_json(cls, fname: str, resolution: tuple = None):
+    def from_json(cls, fname: str, resolution: tuple = None, c_order: bool = True):
         with open(fname, 'r') as file:
             syns = json.load(file)
 
         if resolution is not None:
             syns['resolution'] = resolution
-        return cls.from_dict(syns)
+
+        syns = cls.from_dict(syns)
+
+        if not c_order:
+            syns.transpose_axis()
+            
+        return syns 
 
     @classmethod
     def from_h5(cls, fname: str, resolution: tuple = None, c_order: bool = True):
+        if os.path.getsize(fname) == 0:
+            print(f'synapse file is empty: {fname}')
+            return None
+
         with h5py.File(fname, 'r') as hf:
             if 'pre' in hf.keys():
                 pre = hf['pre']
@@ -209,13 +271,36 @@ class Synapses():
             else:
                 post_confidence = None
 
+            try:
+                users = hf.attrs['users'].split(';')
+            except:
+                users = None
+            #if 'users' in hf.keys():
+            #    users = str(hf['users']).split(';')
+            #else:
+            #    users = None
+            
+            if 'pre_users' in hf.keys():
+                pre_users = np.asarray(hf['pre_users'], dtype=np.int32)
+            else:
+                pre_users = None
+
+            if 'post_users' in hf.keys():
+                post_users = np.asarray(hf['post_users'], dtype=np.int32)
+            else:
+                post_users = None
+
+        syns = cls(
+            pre, post=post, pre_confidence=pre_confidence, 
+            post_confidence=post_confidence, resolution=resolution,
+            users = users,
+            pre_users = pre_users,
+            post_users = post_users,
+        )
         if not c_order:
             # transform to C order
-            pre = pre[:, ::-1]
-            post[:, 1:] = post[:, 1:][:, ::-1]
-
-        return cls(pre, post=post, pre_confidence=pre_confidence, 
-                    post_confidence=post_confidence, resolution=resolution)
+            syns.transpose_axis()
+        return syns
 
     def to_h5(self, fname: str) -> None:
         """save to a HDF5 file
@@ -224,6 +309,7 @@ class Synapses():
             fname (str): the file name to be saved
         """
         assert fname.endswith(".h5") or fname.endswith(".hdf5")
+        print(f'save synapses to {fname}')
         with h5py.File(fname, "w") as hf:
             
             hf['pre'] = self.pre
@@ -240,17 +326,70 @@ class Synapses():
             if self.post_confidence is not None:
                 hf['post_confidence'] = self.post_confidence
 
+            if self.users is not None:
+                hf.attrs['users'] = ';'.join(self.users)
+
+            if self.pre_users is not None:
+                hf['pre_users'] = self.pre_users
+            if self.post_users is not None:
+                hf['post_users'] = self.post_users
+
     @classmethod
     def from_file(cls, fname: str, resolution: tuple = None, c_order: bool = True):
         assert os.path.exists(fname)
         if fname.endswith('.json'):
-            assert c_order
-            return cls.from_json(fname, resolution = resolution)
+            return cls.from_json(fname, resolution = resolution, c_order=c_order)
         elif fname.endswith('.h5'):
             return cls.from_h5(fname, resolution=resolution, c_order=c_order)
         else:
             raise ValueError(f'only support JSON and HDF5 file, but got {fname}')
 
+    def to_neutu_task(self, fname: str, 
+            software_revision: int=4809,
+            description: str = "transformed using chunkflow",
+            file_version: int = 1,
+            body_id: int = None
+        ):
+        """transform to a JSON file as an input to NeuTu.
+        Note that current version only support presynapse. 
+        There is no post-synapses transformed!
+
+        Args:
+            fname (str): file name with extension of .json
+        """
+        assert fname.endswith('.json')
+        task = {
+            'metadata': {
+                "date": time.strftime('%d-%B-%Y %H:%M'),
+                "session path": "",
+                "software revision": software_revision,
+                "description": description,
+                "coordinate system": "dvid",
+                "software": "chunkflow",
+                "file version": file_version,
+                "username": "chunkflow",
+                "software version": chunkflow.version,
+                "computer": "localhost"
+            }
+        }
+
+        if body_id is None:
+            body_id = ""
+
+        data = []
+        for idx in range(self.pre_num):
+            z, y, x = self.pre[idx, :]
+            data.append({
+                "body ID": body_id,
+                "location": [int(x), int(y), int(z)]
+            })
+
+        task['data'] = data
+
+        with open(fname, 'w') as jf:
+            json.dump(task, jf)
+        return
+
     def add_pre(self, pre: np.ndarray, confidence: float = 1.):
         """add some additional pre synapses
 
@@ -267,6 +406,16 @@ class Synapses():
             self.pre_confidence = np.concatenate((self.pre_confidence, confidences), axis=None)
         return self
 
+    def transpose_axis(self):
+        # transform to C order
+        self.pre = self.pre[:, ::-1]
+        self.resolution = self.resolution[::-1]
+        if self.post is not None:
+            self.post[:, 1:] = self.post[:, 1:][:, ::-1]
+        
+    def __len__(self):
+        return self.post_num
+    
     def __eq__(self, other: Synapses) -> bool:
         """compare two synapses.
         Note that we do not compare the confidence here!
@@ -280,8 +429,9 @@ class Synapses():
         if np.array_equal(self.pre, other.pre):
             if self.post is None and other.post is None:
                 return True
-            elif self.post is not None and other.post is not None and np.array_equal(
-                    self.post, other.post):
+            elif self.post is not None and \
+                    other.post is not None and \
+                    np.array_equal(self.post, other.post):
                 return True
             else:
                 return False
@@ -303,7 +453,10 @@ class Synapses():
     
     @property
     def post_num(self) -> int:
-        return self.post.shape[0]
+        if self.post is None:
+            return None
+        else:
+            return self.post.shape[0]
 
     @property
     def pre_bounding_box(self) -> BoundingBox:
@@ -353,10 +506,14 @@ class Synapses():
         # pi2pi = defaultdict(list)
         pi2pi = []
         for idx in range(self.pre_num):
-            # find the post synapses for this presynapse
-            post_indices = np.nonzero(self.post[:, 0]==idx)
-            assert len(post_indices) == 1
-            post_indices = post_indices[0].tolist()
+            if self.post is None:
+                post_indices = None
+            else:
+                # find the post synapses for this presynapse
+                post_indices = np.nonzero(self.post[:, 0]==idx)
+                assert len(post_indices) == 1
+                post_indices = post_indices[0].tolist()
+            
             pi2pi.append(post_indices)
 
         return pi2pi
@@ -368,7 +525,13 @@ class Synapses():
 
     @property
     def distances_from_pre_to_post(self):
-        distances = np.zeros((self.post_num,), dtype=float)
+        """distance from pre to post.
+        the unit is voxel rather than physical.
+
+        Returns:
+            float: the distance
+        """
+        distances = np.zeros((self.post_num,), dtype=np.float32)
         for post_idx in range(self.post_num):
             post = self.post[post_idx, 1:]
             pre_idx = self.post[post_idx, 0]
@@ -407,10 +570,14 @@ class Synapses():
         self.pre = np.delete(self.pre, indices, axis=0)
         if self.pre_confidence is not None:
             self.pre_confidence = np.delete(self.pre_confidence, indices)
+        if self.pre_users is not None:
+            self.pre_users = np.delete(self.pre_users, indices)
 
         if self.post is not None: 
             post_indices = np.isin(self.post[:, 0], indices)
             self.post = np.delete(self.post, post_indices, axis=0)
+            if self.post_users is not None:
+                self.post_users = np.delete(self.post_users, post_indices)
             for idx in range(self.post_num):
                 self.post[idx, 0] = old2new[self.post[idx, 0]]
 
@@ -433,6 +600,106 @@ class Synapses():
 
         self.remove_pre(selected)
 
+    def user_id(self, user: str) -> int:
+        for idx, item in enumerate(self.users):
+            if user == item:
+                return idx 
+        return None
+
+    def post_indices_from_user(self, user: str) -> set:
+        uid = self.user_id(user)
+        assert uid is not None
+        indices = np.nonzero(self.post_users == uid)[0]
+        indices = set(indices.tolist())
+        return indices
+
+    def find_redundent_post(self, num_threshold: int = 15, distance_threshold: float = 50.) -> set:
+        """remove extra number of post synapses. Only keep a maximum number of post synapses.
+
+        Args:
+            num_threshold (int): the maximum number of post synapses kept.
+            distance_threshold (float): the maximum voxel distance from pre to post.
+
+        Return:
+            to_be_removed (set[int]): the post synapse indices to be removed
+        """
+        distances = self.distances_from_pre_to_post
+        assert len(distances) == self.post_num
+
+        # # only remove my own ingestion
+        # predicted_indices = self.post_indices_from_user(user)
+
+        to_be_removed = set()
+        
+        # find the distance over threshold
+        indices = np.nonzero(distances > distance_threshold)
+        assert len(indices) == 1
+        indices = set(indices[0].tolist())
+        to_be_removed = to_be_removed.union(indices)
+        
+        # find the extra number of post synapses
+        for post_indices in self.pre_index2post_indices:
+            if len(post_indices) > num_threshold:
+                # we need to remove some post synapses
+                dis = distances[post_indices]
+
+                if self.post_confidence is not None:
+                    # remove according to confidence
+                    costs = dis / self.post_confidence[post_indices] 
+                else:
+                    # remove according to distance
+                    costs = dis
+                
+                order = np.argsort(costs)
+                post_indices = np.asarray(post_indices, dtype=np.int32)
+                post_indices_to_remove = post_indices[order[num_threshold:]]
+                to_be_removed.union(set(post_indices_to_remove.tolist()))
+
+        # exclude the manually edited post synapses
+        # to_be_removed = to_be_removed.intersection(predicted_indices)
+
+        return to_be_removed
+
+    def find_duplicate_post_on_same_neuron(self, seg: Chunk, distance_threshold: float=10) -> set:
+        """find duplicate post synapses on the same neuron
+        The T-bar could be split to two or three in a long distance
+
+        Args:
+            seg (Chunk): neuron segmentation chunk
+            distance_threshold (float, optional): distance lower than this threshold is regarded as duplicate. Defaults to 10.
+        
+        Return:
+            duplicate_indices (set[int]): a set of post synapse indices that are detected as duplicates.
+        """
+        post_coord = self.post_coordinates - np.asarray(seg.bbox.minpt, dtype=self.post.dtype)
+        kdtree = KDTree(post_coord, leafsize=2)
+        pairs = kdtree.query_pairs(distance_threshold, p=2.0, eps=0, output_type='set')
+
+        distances = self.distances_from_pre_to_post
+
+        duplicated_indices = set()
+        def find_segid(seg: Chunk, coord: np.ndarray):
+            if coord[0] >= seg.shape[0] or coord[1] >= seg.shape[1] or coord[2] >= seg.shape[2]:
+                return None
+            else:
+                return seg[
+                    coord[0],
+                    coord[1],
+                    coord[2]
+                ]
+
+        for idx0, idx1 in pairs:
+            sid0 = find_segid(seg, post_coord[idx0, :])
+            sid1 = find_segid(seg, post_coord[idx1, :])
+            if sid0 is not None and sid1 is not None and sid0 == sid1 and sid0 > 0:
+                if distances[idx0] > distances[idx1]:
+                    duplicated_indices.add(idx0)
+                else:
+                    duplicated_indices.add(idx1)
+        
+        return duplicated_indices
+
+
     def remove_pre_duplicates(self):
         """some presynapses might have same coordinates.
         This is a bug in previous presynapse prediction stage.
@@ -449,4 +716,3 @@ if __name__ == '__main__':
         )
     )
     assert len(synapses.pre_index2post_indices[0]) > 1
-    breakpoint()
\ No newline at end of file
